/*
 * ExactTreePatternAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactTreePatternAutomaton.h"
#include <alphabet/SubtreeWildcardSymbol.h>
#include <tree/Tree.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <automaton/Automaton.h>
#include <automaton/PDA/InputDrivenNPDA.h>

#include <deque>

namespace arbology {

namespace exact {

automaton::Automaton ExactTreePatternAutomaton::construct ( const tree::Tree & tree, const alphabet::Symbol & subtreeWildcard ) {
	return dispatch ( tree.getData ( ), subtreeWildcard );
}

automaton::InputDrivenNPDA ExactTreePatternAutomaton::construct ( const tree::PrefixRankedTree & tree, const alphabet::Symbol & subtreeWildcard ) {
	alphabet::Symbol S = alphabet::symbolFrom ( 'S' );
	automaton::InputDrivenNPDA res ( label::labelFrom ( 0 ), S );

	for ( const alphabet::RankedSymbol & rankedSymbol : tree.getAlphabet ( ) ) {
		alphabet::Symbol symbol ( rankedSymbol );
		res.addInputSymbol ( symbol );
		res.setPushdownStoreOperation ( symbol, std::vector < alphabet::Symbol > ( 1, S ), std::vector < alphabet::Symbol > ( rankedSymbol.getRank ( ).getData ( ), S ) );
	}

	res.addInputSymbol ( subtreeWildcard );
	res.setPushdownStoreOperation ( subtreeWildcard, std::vector < alphabet::Symbol > ( 1, S ), std::vector < alphabet::Symbol > { } );

	int i = 1;
	std::deque < std::pair < unsigned, int > > subtreeJumps;

	for ( const alphabet::RankedSymbol & rankedSymbol : tree.getContent ( ) ) {
		alphabet::Symbol symbol ( rankedSymbol );
		subtreeJumps.push_back ( std::make_pair ( rankedSymbol.getRank ( ).getData ( ), i - 1 ) );

		res.addState ( label::labelFrom ( i ) );
		res.addTransition ( label::labelFrom ( i - 1 ), symbol, label::labelFrom ( i ) );
		res.addTransition ( label::labelFrom ( 0 ), std::move ( symbol ), label::labelFrom ( i ) );

		while ( subtreeJumps.size ( ) && subtreeJumps.back ( ).first == 0 ) {
			res.addTransition ( label::labelFrom ( subtreeJumps.back ( ).second ), subtreeWildcard, label::labelFrom ( i ) );
			subtreeJumps.pop_back ( );
		}

		if ( subtreeJumps.size ( ) ) subtreeJumps.back ( ).first--;

		i++;
	}

	return res;
}

auto ExactTreePatternAutomatonPrefixRankedTree = ExactTreePatternAutomaton::RegistratorWrapper < automaton::InputDrivenNPDA, tree::PrefixRankedTree > ( ExactTreePatternAutomaton::construct );

} /* namespace exact */

} /* namespace arbology */
