/*
 * GlushkovTraversal.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOVTRAVERSAL_H_
#define RTE_GLUSHKOVTRAVERSAL_H_

#include <algorithm>
#include <map>
#include <set>
#include <vector>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElements.h>

#include <exception/CommonException.h>

#include <alphabet/RankedSymbol.h>

namespace rte {

class GlushkovTraversal {
public:
	/**
	 * @param re rte to probe
	 * @return all rteSymbols which can be root of the tree.
	 */
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTE & re );

	/**
	 * @param re rte to probe
	 * @param symbol FormalRTESymbol for which we need the follow(), i.e., Follow(RTE, symbol)
	 * @return all symbols that can follow specific symbol in word
	 */
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTE & re, const alphabet::RankedSymbol & symbol );

	/**
	 * @param re rte to index
	 * @return FormalRTE with indexed elements
	 */
	static FormalRTE index ( const rte::FormalRTE & re );

	/**
	 * @param symbol Glushkov Pair symbol, i.e., SymbolPair of RankedSymbol and integer index
	 * @return RankedSymbol from the pair on input
	 */
	static alphabet::RankedSymbol getSymbolFromGlushkovPair ( const alphabet::RankedSymbol & symbol );

private:
	/**
	 * @return bool true if symbol pointer is in this subtree
	 */
	static bool pos ( const alphabet::RankedSymbol & symbol, const rte::FormalRTE & node );

	static std::rvalue_ref < FormalRTEElement > index ( const rte::FormalRTE & rte, const rte::FormalRTEElement & node, int & i );

	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTEElement & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTEAlternation & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTESubstitution & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTEIteration & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTESymbolAlphabet & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTESymbolSubst & node );
	static std::set < alphabet::RankedSymbol > first ( const rte::FormalRTEEmpty & node );

	static bool pos ( const rte::FormalRTEElement & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTEAlternation & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTESubstitution & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTEIteration & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTESymbolAlphabet & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTESymbolSubst & node, const alphabet::RankedSymbol & symbSearch );
	static bool pos ( const rte::FormalRTEEmpty & node, const alphabet::RankedSymbol & symbSearch );

	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTEElement & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTEAlternation & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTESubstitution & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTEIteration & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTESymbolAlphabet & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTESymbolSubst & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
	static std::set < std::vector < alphabet::RankedSymbol > > follow ( const rte::FormalRTEEmpty & node, const alphabet::RankedSymbol & symbol, const std::set < alphabet::RankedSymbol > & alphabetK, std::map < alphabet::RankedSymbol, std::set < alphabet::RankedSymbol > > & subM );
};

} /* namespace rte */

#endif /* RTE_GLUSHKOVTRAVERSAL_H_ */
